import json
import os
import yaml
import glob 
import numpy as np
formatter = {"Glob": "Global", "NAtl": "NAtl", "Gyre": "SPG"}

# Function to read a JSON file and save it to a dictionary
def json_to_dict(json_files):
    file_name_split = json_files[0].split("/")
    data = {"Description":
            {"Constraint_approach": json_files[0],
            "Constraint region": formatter[file_name_split[-1].split("_")[0]],
            "Constraint variable": "SST",
            "Ensemble size": None,
            "Accumulation years": 9.0}
            }
    dataset_len = []
    for json_file in json_files:
        with open(json_file, 'r') as file:
            d = json.load(file)
        sd = json_file.split(".json")[0].split("_")[-1]
        ls = list(d.keys())
        if "CESM2_r1i1p1f1" in ls:
            ls.remove("CESM2_r1i1p1f1")
        data[f"s{sd}"] = ls
        dataset_len.append(len(ls))
    data["Description"]["Ensemble size"] = float(np.mean(np.array(dataset_len)))
    return data

# Function to save a dictionary to a YAML file
def dict_to_yaml(data, yaml_file):
    with open(yaml_file, 'w') as file:
        yaml.dump(data, file, default_flow_style=False)

# Main function to read JSON and save as YAML
def convert_json_to_yaml():
    outputs = ["/esarchive/scratch/jcos/output/recipe_light_observational_constrains_20220621_095313/work/obs_constrains/tos_s*", "/esarchive/scratch/jcos/output/recipe_decadal_constrains_20220718_053727/work/constr_*/*/"]
    approaches = ["PC", "Befort"]
    regions = ["Glob", "NAtl", "Gyre"]
    for method, output in zip(["OBS", "DP"], outputs):
        for approach in approaches:
            for region in regions:
                json_files=glob.glob(os.path.join(output, approach, "1-9y", f"{region}_*.json"))
                json_files.sort()
                if len(json_files) == 40:
                    # Step 1: Read the JSON file into a dictionary
                    data = json_to_dict(json_files)
                    target_folder = "/esarchive/scratch/cdelgado/aspect_outputs/wp3_d3.1/"
                    target_folder = "/esarchive/scratch/jcos/constraints_paper_figures_data_scritps/ASPECT"
                    
                    yaml_file=os.path.join(target_folder, f"{method}-{approach}" ,f"RankedMembers_accumyears{int(data['Description']['Accumulation years'])}_constraint_region{formatter[region]}_ensemble{int(data['Description']['Ensemble size'])}.yaml")
                    if not os.path.isdir(os.path.join(target_folder, f"{method}-{approach}")):
                        os.makedirs(os.path.join(target_folder, f"{method}-{approach}"))
                    # # Step 2: Save the dictionary to a YAML file
                    dict_to_yaml(data, yaml_file)
                    # print(f"Successfully converted {json_file} to {yaml_file}")
                else:
                    print(f"{method}-{approach}-{region}")

# Usage example
if __name__ == "__main__":
    convert_json_to_yaml()