
import matplotlib.pyplot as plt
import glob
import os
import warnings
import yaml
import json
import logging
import numpy as np
from dask import array as da
import esmvaltool.diag_scripts.shared as e
from esmvaltool.diag_scripts.shared import names, extract_variables, io
from datetime import datetime
import sys
import matplotlib.patches as patches
import matplotlib.colors as mcolors
import glob
import matplotlib.gridspec as gs


current = os.path.dirname(os.path.realpath(__file__))
parent_directory = os.path.dirname(current)
sys.path.append(parent_directory)

logger = logging.getLogger(__name__)
FORMAT = "%(asctime)-15s %(message)s"
logging.basicConfig(format=FORMAT)


startTime = datetime.now()
warnings.filterwarnings("ignore")
config_loops_file = "/esarchive/scratch/jcos/constraints_paper_figures_data_scritps/config_loops.yml"
with open(config_loops_file, "r") as f:
    config_loops = yaml.full_load(f)

submethod = {"PC": "field", "Befort": "time"}
# label={1: {1},2:{1:,2:}, 3:{1:, 2:}}
# Create the dictionary
leters=np.array([f"{chr(65 + i).lower()})" for i in range(2*3)]).reshape(3,2)

dates=np.arange(1970,2001)
fig = plt.figure(figsize=(18.9, 9))
gs = gs.GridSpec(3, 2, wspace=0.005, hspace=0.45)
row = 0
for method in ["OBS","DP"]:
    if method=="DP":
        selection_criterias = ["PC", "Befort"]
    else:
        selection_criterias = ["PC"]
    for selection_criteria in selection_criterias:
        N=30
        if selection_criteria == "Befort":
            N=35
        files = glob.glob(f"/esarchive/scratch/jcos/constraints_paper_figures_data_scritps/files/{method}_corrs_array_{selection_criteria}_best{N}_*.npy")
        files.sort()
        col = 0
        for file in files:
            region = file.split("_")[-1].split(".npy")[0]
            corrs_array = np.load(file)
            ax = plt.subplot(gs[row,col])
            plt.plot(dates, np.median(corrs_array, axis=1), color="black", label="median")
            plt.plot(dates, np.max(corrs_array, axis=1), linestyle="--", color="black", label="max/min")
            plt.plot(dates, np.min(corrs_array, axis=1), linestyle="--", color="black")
            plt.title(f"Constraining metric values for the best {N} members\nmethod: {region} {method}{submethod[selection_criteria]}-selection", fontsize="x-large")
            plt.xticks(fontsize="large")
            plt.yticks(fontsize="large")
            if submethod[selection_criteria] == "time":
                plt.ylim(0,0.25)
                if col == 0:
                    plt.ylabel("mean absolute error / K", fontsize=14)
                else:
                    plt.yticks([])

            elif col == 0:
                plt.ylabel("uncentered r. / 1", fontsize=14)
                plt.ylim(0,1)
                if row == 0:
                    plt.legend(fontsize="large")
            elif col == 1:
                plt.ylim(0,1)
                plt.yticks([])
            plt.text(plt.xlim()[0]+0.01*(plt.xlim()[1]-plt.xlim()[0]), plt.ylim()[-1]-0.10*(plt.ylim()[1]-plt.ylim()[0]), leters[row][col], fontsize="x-large")
            col += 1
        row += 1

plt.savefig(os.path.join(f"/esarchive/scratch/jcos/constraints_paper_figures_data_scritps/plots", f"selection_value-series.png"), bbox_inches='tight')
plt.savefig(os.path.join(f"/esarchive/scratch/jcos/constraints_paper_figures_data_scritps/plots", f"selection_value-series.pdf"), format="pdf", bbox_inches='tight')
