import iris
import matplotlib.pyplot as plt
import glob
import os
import numpy as np
from matplotlib.gridspec import GridSpec
import yaml

path1 = "/esarchive/scratch/jcos/output/recipe_collect_variable_weights_20220728_125836/work/climWIP_window"
region1 = "Glob" 
path2 = "/esarchive/scratch/jcos/output/recipe_collect_variable_weights_20221129_021534/work/climWIP_window"
region2 = "Med" 
path3 = "/esarchive/scratch/jcos/output/recipe_collect_variable_weights_20220905_094315/work/climWIP_window"
region3 = "WMed" 
path4 = "/esarchive/scratch/jcos/output/recipe_collect_variable_weights_20220905_193959/work/climWIP_window"
region4 = "EMed" 

paths = [path1, path2, path3, path4]
regions = [region1, region2, region3, region4]

paths = [path2]
regions = [region2]

# typ="perf"
# typ="indep"
typ="weight"

for typ in [ "weight", "perf", "indep",]:

    for path, region in zip(paths, regions):
        # read csv files
        if typ == "weight":
            w_files = glob.glob(os.path.join(path,"*","weights.csv"))
        elif typ == "indep":
            w_files = glob.glob(os.path.join(path,"*","norm_weights_indep.csv"))
        elif typ == "perf":
            w_files = glob.glob(os.path.join(path,"*","norm_weights_perfor.csv"))

        print(w_files)
        w_files.sort()
        weights = []
        for w_sd in w_files:
            weights.append(np.loadtxt(w_sd, delimiter=","))
        weights = np.array(weights).transpose()

        # read txt files
        m_files = glob.glob(os.path.join(path, "*", "datasets.txt"))
        # m_files.sort()
        members = []
        for i,m_sd in enumerate(m_files):
            with open(m_sd, "r") as f:
                lines = f.readlines()
            if members != [m for m in lines if not m.startswith("OBS")] and i > 0:
                raise ValueError("member order differs")
            members = [m for m in lines if not m.startswith("OBS")]
        dates = np.arange(1961, 2001)
        members = [m.split("CMIP6_")[-1].replace("_", " ").replace("\n","") for m in members]

        sig_files = glob.glob(os.path.join(path,"*","sigma_d.csv"))
        sig_files.sort()
        sigmas = []
        for sig_sd in sig_files:
            sigmas.append(np.loadtxt(sig_sd, delimiter=","))
        sigmas = np.array(sigmas)

        fig = plt.figure(figsize=(10.8,0.11*len(members)), dpi=100)
        gs = GridSpec(1, 1, left=0.26, right=0.99, top=0.965, bottom=0.025)
        ax = plt.subplot(gs[0, 0])
        fill = plt.pcolormesh(dates[9:], members, weights[:,9:], vmin=0, vmax=np.max(weights), cmap=plt.cm.Greys)
        cbar=plt.colorbar(fill, location="right", orientation="vertical", shrink=0.3, cmap=plt.cm.Greys, label="Weight")
        cbar.ax.tick_params(labelsize=10)
        cbar.set_label('Weight', fontsize=12)
        ymin, ymax = plt.ylim()
        models_all = [m.split(" ")[0] for m in members]
        single_model_list=[]
        id_single_model_transition=[]
        for ki, m in enumerate(models_all):
            if m not in single_model_list:
                single_model_list.append(m)
                id_single_model_transition.append(ki)
        id_single_model_transition.append(ki+1)

        upper_bounds = [k-1 for k in id_single_model_transition][1:]
        id_single_model_transition=id_single_model_transition[:-1]
        bounds = np.stack((id_single_model_transition,upper_bounds), axis=1) # ticks to make lines
        real_bounds=[[k[0]-0.5,k[1]+0.5] for k in bounds] # gridpoint separators
        ticks = [np.mean(real_bounds[k])for k in range(len(single_model_list))] 
        plt.xlim(plt.xlim()[0],plt.xlim()[-1])
        plt.ylim(ymin, ymax)

        
        # STACKED
        ticks_stacked = sorted(list(set(np.array(real_bounds).flatten())))
        tick_lengths=np.tile(np.array([.1,.2,.3,.4,.5,.6,.7,.8])*10,1000)[:len(single_model_list)]
        ax.set_yticks(ticks_stacked)
        ax.set_yticklabels(["" for _ in ticks_stacked], fontsize=10)
        for i, (tk,l) in enumerate(zip(ticks, tick_lengths)):
            ax.hlines(tk,plt.xlim()[0]-l-0.4,plt.xlim()[0],color="gray", linestyle="--", alpha=0.5, clip_on=False)
            ax.vlines(plt.xlim()[0]-l-0.4, tk-1, tk+1,color="gray", alpha=0.7, clip_on=False)
            plt.text(1969.1-l, tk, single_model_list[i], rotation=90, va='center', ha='right', fontsize=14) #, fontweight='bold')#, transform=plt.gca().transAxes)
        for bd,real_bd in zip(bounds, real_bounds):
            plt.hlines(real_bd[0],plt.xlim()[0],plt.xlim()[-1], color="black")
            plt.hlines(real_bd[1],plt.xlim()[0],plt.xlim()[-1], color="black")

        for tick in ax.yaxis.get_major_ticks():
            tick.set_pad(15)


        
        post=""

        plt.xticks(fontsize=15)

        # # BRACKET TICKS
        # ax.set_yticks(ticks)
        # ax.set_yticklabels(single_model_list, fontsize=1)
        # for tick in ax.yaxis.get_major_ticks():
        #     tick.set_pad(15)
        # plt.tick_params(axis='y', length=8)
        # plt.xlim(plt.xlim()[0],plt.xlim()[-1])
        # for bd,real_bd in zip(bounds, real_bounds):
        #     ax.vlines(1969.1, bd[0], bd[1],clip_on=False,color="black")
        #     ax.hlines(bd[0],1968.5,1969.1,clip_on=False,color="black")
        #     ax.hlines(bd[1],1968.5,1969.1,clip_on=False,color="black")
        #     plt.hlines(real_bd[0],plt.xlim()[0],plt.xlim()[-1], color="black")
        #     plt.hlines(real_bd[1],plt.xlim()[0],plt.xlim()[-1], color="black")
        # ax.vlines(1969.5, ymin, ymax,clip_on=False,color="black")

        if typ == "weight":
            plt.title("ClimWIP normalised weights for every\nstart date of the evaluation period (1970-2000)", fontsize="large")
            plt.tight_layout()
            plt.savefig(f"/esarchive/scratch/jcos/constraints_paper_figures_data_scritps/plots/{region}_weights.png", bbox_inches="tight")
            plt.savefig(f"/esarchive/scratch/jcos/constraints_paper_figures_data_scritps/plots/{region}_weights.pdf", format="pdf", bbox_inches="tight")
        elif typ == "indep":
            plt.title("B)   ClimWIP normalised independence weights for every\nstart date of the evaluation period (1970-2000)", fontsize="large")
            plt.tight_layout()
            plt.savefig(f"/esarchive/scratch/jcos/constraints_paper_figures_data_scritps/plots/{region}_indepedence_weights.png", bbox_inches="tight")
            plt.savefig(f"/esarchive/scratch/jcos/constraints_paper_figures_data_scritps/plots/{region}_indepedence_weights.pdf", format="pdf", bbox_inches="tight")
        elif typ == "perf":
            plt.title("A)   ClimWIP normalised performance weights for every\nstart date of the evaluation period (1970-2000)", fontsize="large")
            plt.tight_layout()
            plt.savefig(f"/esarchive/scratch/jcos/constraints_paper_figures_data_scritps/plots/{region}_performance_weights.png", bbox_inches="tight")
            plt.savefig(f"/esarchive/scratch/jcos/constraints_paper_figures_data_scritps/plots/{region}_performance_weights.pdf", format="pdf", bbox_inches="tight")

    # long_models = [m.split(" ")[0] for m in members]
    # models = np.array(sorted(list(set(long_models))))
    # model_w=np.zeros((len(models), weights.shape[-1]))
    # lengths = []
    # for j, model in enumerate(models):
    #     model_sum = [] 
    #     mem_count = 1
    #     for i, member in enumerate(members):
    #         if len(model_sum)==0 and model == member.split(" ")[0]:
    #             model_sum = weights[i].copy()
    #         elif model == member.split(" ")[0]:
    #             model_sum += weights[i]
    #             mem_count += 1
    #     lengths.append(mem_count)
    #     model_w[j] = model_sum


    # percent = np.sum(model_w, axis=1)
    # # percent = np.sum(model_w, axis=1)/lengths
    # norm_percent = percent/np.sum(percent)*100
    # # models which represent 70% of the slection taking into account all start dates.
    # threshold=sorted(norm_percent)[::-1][np.where(np.cumsum(sorted(norm_percent)[::-1])<70)[0][-1]]
    # model_ids = np.where(norm_percent>threshold)
    # most_selected_models = models[model_ids]
    # all_colors = {model: color for model, color in zip(models,plt.cm.get_cmap('hsv', len(models))(range(len(models))))}
    # # sort the selected models according to their values and collect it into a dict
    # best_models_dict=dict(sorted({str(m): float(p) for m, p in zip(most_selected_models, norm_percent[model_ids])}.items(), key=lambda item: item[1], reverse=True))
    # with open(f"/esarchive/scratch/jcos/figures/wcwip/common_framework/test/{region}_best_models.yml", "w") as f:
    #     yaml.dump(best_models_dict, f, default_flow_style=True)
    # colors = [all_colors[c] for c in best_models_dict]
    # fig, ax = plt.subplots()
    # _, labels, _ = ax.pie(list(best_models_dict.values()), labels=list(best_models_dict.keys()), autopct='%1.1f%%', startangle=90, colors=colors)
    # plt.setp(labels, **{'weight':'bold'})
    # # centre_circle = plt.Circle((0,0),0.70,fc='white')
    # # fig = plt.gcf()
    # # fig.gca().add_artist(centre_circle)
    # plt.title(f"Proportion of models explainin 70% of\nthe ensemble for start dates 1961:2000\nusing ClimWIP based on the {region} region")
    # ax.axis("equal")
    # plt.tight_layout()
    # if typ == "weight":
    #     plt.savefig(f"/esarchive/scratch/jcos/figures/wcwip/common_framework/test/{region}_best_pie.png")
    # elif typ == "indep":
    #     plt.savefig(f"/esarchive/scratch/jcos/figures/wcwip/common_framework/test/{region}_indepedence_best_pie.png")
    # elif typ == "perf":
    #     plt.savefig(f"/esarchive/scratch/jcos/figures/wcwip/common_framework/test/{region}_performance_best_pie.png")

    # plt.figure(figsize=(13,25), dpi=200)
    # fill = plt.pcolormesh(dates, models, model_w)
    # plt.colorbar(fill, location="right", orientation="vertical", shrink=0.3, cmap=plt.cm.Greys, label="Weight")
    # plt.tight_layout()
    # if typ == "weight":
    #     plt.savefig(f"/esarchive/scratch/jcos/figures/wcwip/common_framework/test/{region}_model_weights.png")
    # elif typ == "indep":
    #     plt.savefig(f"/esarchive/scratch/jcos/figures/wcwip/common_framework/test/{region}_model_indepedence_weights.png")
    # elif typ == "perf":
    #     plt.savefig(f"/esarchive/scratch/jcos/figures/wcwip/common_framework/test/{region}_model_performance_weights.png")
